// Copyright 2011 Google Inc.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// * Redistributions of source code must retain the above copyright
//   notice, this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the
//   documentation and/or other materials provided with the distribution.
// * Neither the name of Google Inc. nor the names of its contributors
//   may be used to endorse or promote products derived from this software
//   without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// \file engine/plain_iface/test_program.hpp
/// Implementation of test programs that follow the plain interface.

#if !defined(ENGINE_PLAIN_IFACE_TEST_PROGRAM_HPP)
#define ENGINE_PLAIN_IFACE_TEST_PROGRAM_HPP

#include "engine/test_program.hpp"
#include "utils/datetime.hpp"
#include "utils/optional.hpp"

namespace engine {
namespace plain_iface {


namespace detail {


/// The default timeout value for test cases that do not provide one.
/// \todo We should not be doing this; see issue 5 for details.
extern utils::datetime::delta default_timeout;


}  // namespace detail


/// Representation of a plain test program.
class test_program : public base_test_program {
    struct impl;

    /// Pointer to the shared internal implementation.
    std::tr1::shared_ptr< impl > _pimpl;

    test_cases_vector load_test_cases(void) const;

public:
    test_program(const utils::fs::path&, const utils::fs::path&,
                 const std::string&,
                 const utils::optional< utils::datetime::delta >&);
    ~test_program(void);

    const utils::datetime::delta& timeout(void) const;
};


}  // namespace plain_iface
}  // namespace engine

#endif  // !defined(ENGINE_PLAIN_IFACE_TEST_PROGRAM_HPP)
